#include "include/sprite.h"
#include <tonc.h>

extern OBJ_ATTR sprite_list[];
extern int sprite_count;
extern const SimplexSprite simplexSpr[];
extern const ComplexSprite complexSpr[];

void draw_sprite_simplex(int sprite,int pal,int x,int y){
  const SimplexSprite *spr = &simplexSpr[sprite];
  OBJ_ATTR *sprlist = &sprite_list[sprite_count];
  x -= spr->origin_x;
  y -= spr->origin_y;
  sprlist->attr0 = spr->data.attr0;
  sprlist->attr1 = spr->data.attr1;
  sprlist->attr2 = spr->data.attr2 | (pal << 12);
  obj_set_pos(sprlist,x,y);
  sprite_count++;
}

static void load_image(const u32 * src,const Image *img){
  const Piece *pie;
  int i;
  for (i = 0; i < img->piece_count; i++){
    pie = (img->piece_start + i);
	memcpy32(tile_mem[4] + (pie->data.attr2 & 0x03FF),
	  src + pie->src_offset,
	  pie->length);
  }
}

static void draw_image(const ComplexSprite *spr,
                       const Image *img,int pal,int x,int y){
  OBJ_ATTR *sprlist;
  const Piece *pie;
  int i,dx;
  for (i = 0; i < img->piece_count; i++){
    sprlist = &sprite_list[sprite_count];
    pie = (img->piece_start + i);
	sprlist->attr0 = pie->data.attr0;
	sprlist->attr1 = pie->data.attr1;
	dx = (sprlist->attr1 & 0x01FF);
	sprlist->attr2 = pie->data.attr2 | (pal << 12);
	if (spr->flipped){
	  sprlist->attr1 |= 0x1000;
	  dx = spr->width - obj_get_width(&(pie->data)) - dx;
	}
	obj_set_pos(sprlist,x + dx,y + (sprlist->attr0 & 0x00FF));
	sprite_count++;
  }
}
  
void init_sprite_complex(Animat *ani,int sprite){
  const ComplexSprite *spr;
  if (ani->sprite_index != sprite){
    spr = &complexSpr[sprite];
    if (spr->init_image)
	  load_image(spr->image_src,spr->init_image);
	ani->sprite_index = sprite;
	ani->image_index = 0;
	ani->image_incrm = 0;
  }
}

void draw_sprite_complex(Animat *ani,int x,int y){
  const ComplexSprite *spr = &complexSpr[ani->sprite_index];
  const Image *img = spr->draw_image;
  int image = ani->image_index;
  int pal = ani->palette;
  x -= spr->origin_x;
  y -= spr->origin_y;
  if (spr->init_image)
    draw_image(spr,spr->init_image,pal,x,y);
  if (img){
    int incrm_limit;
    if (spr->anim_image){
	  const int * anim_index = spr->anim_image + (image << 1);
      img += *anim_index;
	  incrm_limit = *(anim_index + 1);
	}
	else{
	  img += image;
	  incrm_limit = 24;
	}
	if (ani->image_incrm == 0)
	  load_image(spr->image_src,img);
	draw_image(spr,img,pal,x,y);
	if (ani->image_speed)
	  ani->image_incrm += ani->image_speed;
	else
	  ani->image_incrm = 1;
	if (ani->image_incrm == incrm_limit){
   	  ani->image_incrm = 0;
	  ani->image_index++;
	}
	if (ani->image_index == spr->draw_count)
	  ani->image_index = 0;
  }
}